import junit.framework.TestCase;

public class LLEditableQueueTester extends TestCase {

	/**
	 * Tests the empty queue is set up properly. 
	 *
	 */
	public void testEmptyQueue() {
		LLEditableQueue<Integer> q = new LLEditableQueue<Integer>();
		assertEquals(0, q.size());	
		assertTrue(q.isEmpty()); 
	}
	
	/**
	 * Tests basic enqueuing dequeuing of 1 element 
	 *
	 */
	public void testEnqueueDequeue1() {
		LLEditableQueue<Integer> q = new LLEditableQueue<Integer>();
		q.enqueue(5); 		 
		assertEquals(5,(int)q.dequeue());
		assertEquals(0, q.size());
		assertTrue(q.isEmpty()); 
	}
	
	/**
	 * Tests enqueuing dequeing of several elements
	 * 
	 */
	public void testEnqueueDequeue2() {
		LLEditableQueue<String> q = new LLEditableQueue<String>();
		q.enqueue("hello"); 
		q.enqueue("goodbye"); 
		q.enqueue("knock knock"); 
		q.enqueue("who's there");
		assertEquals(4, q.size()); 
		assertEquals("hello", q.dequeue());
		assertEquals(3, q.size()); 
		assertEquals("goodbye", q.dequeue()); 
		assertEquals("knock knock", q.dequeue()); 
		assertEquals("who's there", q.dequeue()); 
		assertEquals(0, q.size());
		assertTrue(q.isEmpty()); 		
	}
	
	/**
	 * Tests enqueuing dequeing (interleaved) of several elements
	 * 
	 */
	public void testEnqueueDequeue3() {
		LLEditableQueue<String> q = new LLEditableQueue<String>();
		q.enqueue("hello"); 
		q.enqueue("goodbye"); 
		assertEquals(2, q.size());
		assertEquals("hello", q.dequeue());
		assertEquals(1, q.size()); 
		assertEquals("goodbye", q.dequeue());
		q.enqueue("knock knock"); 
		q.enqueue("who's there");
		assertEquals(2, q.size()); 
 
		assertEquals("knock knock", q.dequeue()); 
		assertEquals("who's there", q.dequeue()); 
		assertEquals(0, q.size());
		assertTrue(q.isEmpty()); 		
	}
	
	/**
	 * Tests the front() method
	 */
	public void testFront() {
		LLEditableQueue<String> q = new LLEditableQueue<String>();
		q.enqueue("hello"); 
		q.enqueue("goodbye"); 
		q.enqueue("knock knock"); 
		q.enqueue("who's there"); 
		assertEquals("hello", q.front());
		q.dequeue(); 
		assertEquals("goodbye", q.front()); 
		q.dequeue();
		assertEquals("knock knock", q.front());
		q.dequeue();
		assertEquals("who's there", q.front());
		q.dequeue();		
	}
	
	/**
	 * Tests the remove() method with unique objects 
	 */
	public void testRemove1() {
		LLEditableQueue<Object> q = new LLEditableQueue<Object>(); 
		Object[] objs = new Object[10]; 
		for (int i = 0; i < objs.length; i++) {
			objs[i] = new Object();
			q.enqueue(objs[i]); 
		}
		// remove every other object 
		for (int i = 0; i < objs.length; i += 2) {
			assertTrue(q.remove(objs[i])); 
		}
		assertEquals(5, q.size()); 
		// make sure the proper objects remain
		for (int i = 1; i < objs.length; i += 2) {
			assertEquals(objs[i], q.dequeue()); 
		}
		
	}
	
	/**
	 * Tests the remove() method with duplicate objects
	 */
	public void testRemove2() {
		LLEditableQueue<Integer> q = new LLEditableQueue<Integer>();
		q.enqueue(1); 
		q.enqueue(2); 
		q.enqueue(3); 
		q.enqueue(3); 
		q.enqueue(2);
		q.enqueue(1); 
		
		q.remove(2); 
		assertEquals(5, q.size()); 
		assertEquals(1, (int)q.dequeue()); 
		assertEquals(3, (int)q.dequeue()); 
		assertEquals(3, (int)q.dequeue()); 
		assertEquals(2, (int)q.dequeue()); 
		assertEquals(1, (int)q.dequeue()); 
		
	}
	
	
}
