
import junit.framework.TestCase;

/**
 * A JUnit test case for the QStack class.
 */
public class QStackTester extends TestCase {

	/**
	 * Test that the empty stack is set up properly
	 */
	public void testEmptyStack() {
		QStack<Integer> qstack = new QStack<Integer>();
		assertTrue(qstack.isEmpty());
		assertEquals(0, qstack.size());
	}

	/**
	 * A simple test of push and pop
	 *
	 */
	public void testPushPop1() {
		QStack<Integer> qstack = new QStack<Integer>();

		assertTrue(qstack.isEmpty());
		qstack.push(1);
		assertFalse(qstack.isEmpty());

		assertEquals(1, (int) qstack.pop());
		assertTrue(qstack.isEmpty());		
	}
	
	/**
	 * A more complicated test case to ensure that push and pop were implemented
	 * correctly.
	 * 
	 */
	public void testPushPop2() {
		QStack<Integer> qstack = new QStack<Integer>();
		
		qstack.push(2);
		qstack.push(3);
		qstack.push(5);

		assertEquals(3, qstack.size());
		assertEquals(5, (int) qstack.pop());
		assertEquals(3, (int) qstack.pop());
		assertEquals(2, (int) qstack.pop());
	}
	
	/**
	 * A more complicated test case to ensure that push and pop were implemented
	 * correctly.
	 * 
	 */
	public void testPushPop3() {
		QStack<String> qstack = new QStack<String>(); 
		
		qstack.push("hello"); 
		qstack.push("world"); 
		qstack.push("goodbye"); 
		qstack.push("knock"); 
		qstack.push("knock");
		assertEquals(5, qstack.size()); 
		
		assertEquals("knock", qstack.pop()); 
		assertEquals("knock", qstack.pop());
		assertEquals("goodbye", qstack.pop());
		assertEquals("world", qstack.pop());
		assertEquals("hello", qstack.pop());
	}
	
	/**
	 * A more complicated test case to ensure that push and pop were implemented
	 * correctly. This test interleaves pushing and popping. 
	 * 
	 */
	public void testPushPop4() {
		QStack<Integer> qstack = new QStack<Integer>(); 
		qstack.push(1); 
		qstack.push(2); 
		assertEquals(2, (int)qstack.pop()); 
		qstack.push(2);
		assertEquals(2, (int)qstack.pop());
	    qstack.push(2); 
	    qstack.push(3); 
	    assertEquals(3, (int)qstack.pop()); 
	    assertEquals(2, (int)qstack.pop()); 
	    assertEquals(1, (int)qstack.pop()); 
	}


}
