import junit.framework.TestCase;

public class RoundRobinRobotTester extends TestCase {

	/**
	 * Test case that runs an empty robot. 
	 *
	 */
	public void testEmptyRobot() {
		RoundRobinRobot rr = new RoundRobinRobot(15); 
		rr.run(12); 
		assertEquals(0, rr.getRunningTime()); 
		
		rr.run(20); 
		assertEquals(0, rr.getRunningTime()); 
	}
	
	/**
	 * Test case that runs robot with 1 task 
	 *
	 */
	public void testSingleTask() {
		
		
		RoundRobinRobot rr = new RoundRobinRobot(15);
		Instruction m0 = new ReturnInstruction(); 
		Function main = new Function(); 
		main.addInstruction(m0); 
		Task t1 = new ModelSolutionTask(main); 
		rr.addTask(t1); 
		
		// run less than the quantum
		rr.run(12);
		assertEquals(0, rr.getRunningTime()); 
		
		TaskState ts = t1.getState();
		check(ts, false, 0, main, 0); 
		
		rr.run(15); 
		ts = t1.getState(); 
		check(ts, true, 10, main, -1); 
		assertEquals(10, rr.getRunningTime()); 
		
	}
	
	
	/**
	 * Performs assertions for checking the TaskState was set properly. 
	 * 
	 * @param ts task state
	 * @param done whether task should be done 
	 * @param runningTime the current running time of the task 
	 * @param curFunction the current function executing 
	 * @param index the instruction to be next executed
	 */
	private void check(TaskState ts, boolean done, long runningTime,
			Function curFunction, int index) {
		assertEquals(done, ts.isDone());
		assertEquals(runningTime, ts.getRunningTime());
		if (!done) {
			assertEquals(curFunction, ts.getCurrentFunction());
			assertEquals(index, ts.getInstructionIndex());
		}
	}
	
	
}
